# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
from typing import Any, Callable, Dict, IO, Iterable, Optional, TypeVar, Union, overload
import urllib.parse

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from .._serialization import Serializer
from .._vendor import _convert_request

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_list_by_endpoint_resource_request(resource_uri: str, endpoint_name: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-03-15"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "resourceUri": _SERIALIZER.url("resource_uri", resource_uri, "str", skip_quote=True),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str", skip_quote=True),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_request(
    resource_uri: str, endpoint_name: str, service_configuration_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-03-15"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations/{serviceConfigurationName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "resourceUri": _SERIALIZER.url("resource_uri", resource_uri, "str", skip_quote=True),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str", skip_quote=True),
        "serviceConfigurationName": _SERIALIZER.url(
            "service_configuration_name", service_configuration_name, "str", skip_quote=True
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_create_orupdate_request(
    resource_uri: str, endpoint_name: str, service_configuration_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-03-15"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations/{serviceConfigurationName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "resourceUri": _SERIALIZER.url("resource_uri", resource_uri, "str", skip_quote=True),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str", skip_quote=True),
        "serviceConfigurationName": _SERIALIZER.url(
            "service_configuration_name", service_configuration_name, "str", skip_quote=True
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_update_request(
    resource_uri: str, endpoint_name: str, service_configuration_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-03-15"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations/{serviceConfigurationName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "resourceUri": _SERIALIZER.url("resource_uri", resource_uri, "str", skip_quote=True),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str", skip_quote=True),
        "serviceConfigurationName": _SERIALIZER.url(
            "service_configuration_name", service_configuration_name, "str", skip_quote=True
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PATCH", url=_url, params=_params, headers=_headers, **kwargs)


def build_delete_request(
    resource_uri: str, endpoint_name: str, service_configuration_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-03-15"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations/{serviceConfigurationName}",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "resourceUri": _SERIALIZER.url("resource_uri", resource_uri, "str", skip_quote=True),
        "endpointName": _SERIALIZER.url("endpoint_name", endpoint_name, "str", skip_quote=True),
        "serviceConfigurationName": _SERIALIZER.url(
            "service_configuration_name", service_configuration_name, "str", skip_quote=True
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, headers=_headers, **kwargs)


class ServiceConfigurationsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.hybridconnectivity.HybridConnectivityMgmtClient`'s
        :attr:`service_configurations` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_by_endpoint_resource(
        self, resource_uri: str, endpoint_name: str, **kwargs: Any
    ) -> Iterable["_models.ServiceConfigurationResource"]:
        """Lists of all the services associated with endpoint resource.

        API to enumerate registered services in service configurations under a Endpoint Resource.

        :param resource_uri: The fully qualified Azure Resource manager identifier of the resource to
         be connected. Required.
        :type resource_uri: str
        :param endpoint_name: The endpoint name. Required.
        :type endpoint_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either ServiceConfigurationResource or the result of
         cls(response)
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.ServiceConfigurationList] = kwargs.pop("cls", None)

        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                request = build_list_by_endpoint_resource_request(
                    resource_uri=resource_uri,
                    endpoint_name=endpoint_name,
                    api_version=api_version,
                    template_url=self.list_by_endpoint_resource.metadata["url"],
                    headers=_headers,
                    params=_params,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("ServiceConfigurationList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    list_by_endpoint_resource.metadata = {
        "url": "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations"
    }

    @distributed_trace
    def get(
        self, resource_uri: str, endpoint_name: str, service_configuration_name: str, **kwargs: Any
    ) -> _models.ServiceConfigurationResource:
        """Gets the details about the service to the resource.

        :param resource_uri: The fully qualified Azure Resource manager identifier of the resource to
         be connected. Required.
        :type resource_uri: str
        :param endpoint_name: The endpoint name. Required.
        :type endpoint_name: str
        :param service_configuration_name: The service name. Required.
        :type service_configuration_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ServiceConfigurationResource or the result of cls(response)
        :rtype: ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.ServiceConfigurationResource] = kwargs.pop("cls", None)

        request = build_get_request(
            resource_uri=resource_uri,
            endpoint_name=endpoint_name,
            service_configuration_name=service_configuration_name,
            api_version=api_version,
            template_url=self.get.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ServiceConfigurationResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {
        "url": "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations/{serviceConfigurationName}"
    }

    @overload
    def create_orupdate(
        self,
        resource_uri: str,
        endpoint_name: str,
        service_configuration_name: str,
        service_configuration_resource: _models.ServiceConfigurationResource,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ServiceConfigurationResource:
        """Create or update a service in serviceConfiguration for the endpoint resource.

        :param resource_uri: The fully qualified Azure Resource manager identifier of the resource to
         be connected. Required.
        :type resource_uri: str
        :param endpoint_name: The endpoint name. Required.
        :type endpoint_name: str
        :param service_configuration_name: The service name. Required.
        :type service_configuration_name: str
        :param service_configuration_resource: Service details. Required.
        :type service_configuration_resource:
         ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ServiceConfigurationResource or the result of cls(response)
        :rtype: ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_orupdate(
        self,
        resource_uri: str,
        endpoint_name: str,
        service_configuration_name: str,
        service_configuration_resource: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ServiceConfigurationResource:
        """Create or update a service in serviceConfiguration for the endpoint resource.

        :param resource_uri: The fully qualified Azure Resource manager identifier of the resource to
         be connected. Required.
        :type resource_uri: str
        :param endpoint_name: The endpoint name. Required.
        :type endpoint_name: str
        :param service_configuration_name: The service name. Required.
        :type service_configuration_name: str
        :param service_configuration_resource: Service details. Required.
        :type service_configuration_resource: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ServiceConfigurationResource or the result of cls(response)
        :rtype: ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_orupdate(
        self,
        resource_uri: str,
        endpoint_name: str,
        service_configuration_name: str,
        service_configuration_resource: Union[_models.ServiceConfigurationResource, IO],
        **kwargs: Any
    ) -> _models.ServiceConfigurationResource:
        """Create or update a service in serviceConfiguration for the endpoint resource.

        :param resource_uri: The fully qualified Azure Resource manager identifier of the resource to
         be connected. Required.
        :type resource_uri: str
        :param endpoint_name: The endpoint name. Required.
        :type endpoint_name: str
        :param service_configuration_name: The service name. Required.
        :type service_configuration_name: str
        :param service_configuration_resource: Service details. Is either a
         ServiceConfigurationResource type or a IO type. Required.
        :type service_configuration_resource:
         ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ServiceConfigurationResource or the result of cls(response)
        :rtype: ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ServiceConfigurationResource] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(service_configuration_resource, (IOBase, bytes)):
            _content = service_configuration_resource
        else:
            _json = self._serialize.body(service_configuration_resource, "ServiceConfigurationResource")

        request = build_create_orupdate_request(
            resource_uri=resource_uri,
            endpoint_name=endpoint_name,
            service_configuration_name=service_configuration_name,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.create_orupdate.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if response.status_code == 200:
            deserialized = self._deserialize("ServiceConfigurationResource", pipeline_response)

        if response.status_code == 201:
            deserialized = self._deserialize("ServiceConfigurationResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    create_orupdate.metadata = {
        "url": "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations/{serviceConfigurationName}"
    }

    @overload
    def update(
        self,
        resource_uri: str,
        endpoint_name: str,
        service_configuration_name: str,
        service_configuration_resource: _models.ServiceConfigurationResourcePatch,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ServiceConfigurationResource:
        """Update the service details in the service configurations of the target resource.

        :param resource_uri: The fully qualified Azure Resource manager identifier of the resource to
         be connected. Required.
        :type resource_uri: str
        :param endpoint_name: The endpoint name. Required.
        :type endpoint_name: str
        :param service_configuration_name: The service name. Required.
        :type service_configuration_name: str
        :param service_configuration_resource: Service details. Required.
        :type service_configuration_resource:
         ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResourcePatch
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ServiceConfigurationResource or the result of cls(response)
        :rtype: ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update(
        self,
        resource_uri: str,
        endpoint_name: str,
        service_configuration_name: str,
        service_configuration_resource: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ServiceConfigurationResource:
        """Update the service details in the service configurations of the target resource.

        :param resource_uri: The fully qualified Azure Resource manager identifier of the resource to
         be connected. Required.
        :type resource_uri: str
        :param endpoint_name: The endpoint name. Required.
        :type endpoint_name: str
        :param service_configuration_name: The service name. Required.
        :type service_configuration_name: str
        :param service_configuration_resource: Service details. Required.
        :type service_configuration_resource: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ServiceConfigurationResource or the result of cls(response)
        :rtype: ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update(
        self,
        resource_uri: str,
        endpoint_name: str,
        service_configuration_name: str,
        service_configuration_resource: Union[_models.ServiceConfigurationResourcePatch, IO],
        **kwargs: Any
    ) -> _models.ServiceConfigurationResource:
        """Update the service details in the service configurations of the target resource.

        :param resource_uri: The fully qualified Azure Resource manager identifier of the resource to
         be connected. Required.
        :type resource_uri: str
        :param endpoint_name: The endpoint name. Required.
        :type endpoint_name: str
        :param service_configuration_name: The service name. Required.
        :type service_configuration_name: str
        :param service_configuration_resource: Service details. Is either a
         ServiceConfigurationResourcePatch type or a IO type. Required.
        :type service_configuration_resource:
         ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResourcePatch or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ServiceConfigurationResource or the result of cls(response)
        :rtype: ~azure.mgmt.hybridconnectivity.models.ServiceConfigurationResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ServiceConfigurationResource] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(service_configuration_resource, (IOBase, bytes)):
            _content = service_configuration_resource
        else:
            _json = self._serialize.body(service_configuration_resource, "ServiceConfigurationResourcePatch")

        request = build_update_request(
            resource_uri=resource_uri,
            endpoint_name=endpoint_name,
            service_configuration_name=service_configuration_name,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.update.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ServiceConfigurationResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update.metadata = {
        "url": "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations/{serviceConfigurationName}"
    }

    @distributed_trace
    def delete(  # pylint: disable=inconsistent-return-statements
        self, resource_uri: str, endpoint_name: str, service_configuration_name: str, **kwargs: Any
    ) -> None:
        """Deletes the service details to the target resource.

        :param resource_uri: The fully qualified Azure Resource manager identifier of the resource to
         be connected. Required.
        :type resource_uri: str
        :param endpoint_name: The endpoint name. Required.
        :type endpoint_name: str
        :param service_configuration_name: The service name. Required.
        :type service_configuration_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        request = build_delete_request(
            resource_uri=resource_uri,
            endpoint_name=endpoint_name,
            service_configuration_name=service_configuration_name,
            api_version=api_version,
            template_url=self.delete.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    delete.metadata = {
        "url": "/{resourceUri}/providers/Microsoft.HybridConnectivity/endpoints/{endpointName}/serviceConfigurations/{serviceConfigurationName}"
    }
